# Benötigte Pakete 
library(fitdistrplus)
library(ggplot2)

# Bevölkerung am 1. 1. 2024, monatlich um 3.490,5 gestiegen (Durchschnitt bis zum 1. 7. 2024)
# Quelle: Statistik Austria
bevölkerung = seq(9158750, 9158750 + 12 * 3490.5, 3490.5)
halbbev = bevölkerung/2

# Katholiken am 31. 12. 2023
# Quelle: Katholische Kirche
kath = 4638842

# Verlust an Mitgliedern in den letzten 20 Jahren
# Quelle: Katholische Kirche, abgeleitet aus angegebener Mitgliederzahl in den jährlichen Daten
jahresverlust = c(50160, 38343, 31557, 32417, 36166, 51191, 84586, 56122, 51698, 54612, 51816, 
                  57275, 52522, 52531, 58153, 68405, 76037, 75967, 94389, 91421)

verlust_nach_jahr = data.frame(
    jahr = 2004:2023,
    rückgang = jahresverlust
)

ggplot(verlust_nach_jahr, aes(jahr, rückgang)) + geom_line(linewidth=0.9) + 
    geom_smooth(linewidth=0.4) +
    labs(
        x = "Jahr", y = "Mitgliederverlust",
        title = "Mitgliederverlust der römisch-katholischen Kirche pro Jahr, 2004-2023"
    ) +
    scale_y_continuous(labels = scales::label_number(big.mark = ".", decimal.mark = ","))

    
#ggsave("/tmp/verlustnachjahr.png", width=8, height=4) 

# So viele Jahre berücksichtigen wir
jahre = 3

# Wir arbeiten mit den letzten x Jahren, ...
jv = jahresverlust[(length(jahresverlust) - jahre + 1):length(jahresverlust)]
# ... und gewichten die späteren Jahre jeweils höher, indem wir sie wiederholen
for (j in (2:jahre)) {
    # Nächste Zeile auskommentieren, um nicht zu gewichten
    jv = c(jv, rep(jv[j], j - 1))
}

# Manuelle Eingabe einer Schätzung für 2024 - auskommentieren, wenn nicht gewünscht
#jv = c(jv, 95000)
    
# Daten visualisieren
qplot(jv)

# Log-normal
fit <- fitdist(jv, "lnorm")
fit$aic
fit$estimate

plot(fit)

# Liefert die Wahrscheinlichkeit für den angegebenen Wert aus der erstellten Verteilung
ws <- function(wert) {
    return(plnorm(wert, meanlog = fit$estimate["meanlog"], sdlog=fit$estimate["sdlog"]))
}

# Fragestellung: Wie ist die Wahrscheinlichkeit für jeden Monat, dass der Mitgliederverlust
# in diesem Monat ausgereicht hat, um die Hälfte der Bevölkerung zu erreichen?
monate = seq(1:12)

for (mon in monate) {
    
    # Halbe Bevölkerung nach mon Monaten
    hbevmon = round(halbbev[1 + mon])
    
    # Differenz zu den Katholiken vom Jahresanfang
    diffkath = kath - hbevmon
    
    # Wie groß müsste der jährliche Bevölkerungsverlust sein, damit es sich schon in diesem Monat ausgeht?
    diffkathmon = diffkath * (12 / mon)
    
    # Errechnete Daten ausgeben
    cat(paste("Berücksichtige ", jahre, " Jahre, ",
              "Monat: ", mon,
              ", notwendiger Rückgang: ", round(diffkathmon), 
              ", Wahrscheinlichkeit: ", format((1.0 - ws(diffkathmon)) * 100, digits = 2), " %",
              sep = ""),
              fill=TRUE)
}
